<?php

namespace Drupal\layoutbuilder_extras_live_update;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Form\FormStateInterface;
use Drupal\layout_builder\Controller\LayoutRebuildTrait;

/**
 * Callbacks for the ajax live reloading.
 */
class LayoutBuilderExtras {

  use LayoutRebuildTrait;

  /**
   * Saves the form. So that the settings that you change are saved.
   *
   * Because on move HTML gets updates and shit gets lost.
   *
   * @param array $form
   *   Form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   An ajax response to maybe change the content.
   */
  public function ajaxSave(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\layoutbuilder_extras\Form\LayoutBuilderExtrasConfigureSectionForm $configureSectionForm */
    $configureSectionForm = $form_state->getBuildInfo()['callback_object'];

    if ($configureSectionForm->isUpdate()) {
      return $this->updateLayoutAndReturnAjaxResponse($form_state, $form);
    }
    else {
      // Return empty response.
      return new AjaxResponse();
    }

  }

  /**
   * Validate and submitted configure section form.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array $form
   *   The form.
   *
   * @return \Drupal\layout_builder\Form\ConfigureSectionForm
   *   A configure section form that is validated an submitted.
   */
  private function validateAndSubmitConfigureForm(FormStateInterface $form_state, array &$form) {
    /** @var \Drupal\layout_builder\Form\ConfigureSectionForm $configureSectionForm */
    $configureSectionForm = $form_state->getBuildInfo()['callback_object'];
    // Needed for visual change.
    $configureSectionForm->validateForm($form, $form_state);
    $configureSectionForm->submitForm($form, $form_state);
    return $configureSectionForm;
  }

  /**
   * Saves the block form. So that the settings/fields that you change are saved.
   *
   * @param array $form
   *   Form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   An ajax response to maybe change the content.
   */
  public function blockAjaxSave(array &$form, FormStateInterface $form_state) {
    return $this->updateLayoutAndReturnAjaxResponse($form_state, $form);
  }

  /**
   * Updates the forms and returns an ajax response to update it.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param array $form
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  private function updateLayoutAndReturnAjaxResponse(FormStateInterface $form_state, array $form): AjaxResponse {
    // Needed for change in TPL.
    /** @var \Drupal\layout_builder\SectionStorageInterface $section_storage */
    $section_storage = $form_state->getBuildInfo()['args'][0];
    $this->validateAndSubmitConfigureForm($form_state, $form);

    return $this->rebuildLayout($section_storage);
  }

  /**
   * Used for block forms like our Video block.
   *
   * @param array $elements
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return mixed
   */
  public function processBlockForm(array &$elements, FormStateInterface $form_state, $form, $elementTypeManager = NULL, $definitions = NULL) {
    // @todo is the best way?
    if (empty($elementTypeManager)) {
      /** @var \Drupal\layoutbuilder_extras_live_update\ElementTypeManagerInterface $elementTypeManager */
      $elementTypeManager
        = \Drupal::service('layoutbuilder_extras_live_update.element_type.manager');
      $definitions = $elementTypeManager->getDefinitions();
    }

    foreach ($elements as $key => &$element) {
      $isApplicable =
        LayoutBuilderExtrasLiveUpdateHelper::isElementApplicable($key, $element);

      if (is_array($element)) {
        $this->processBlockForm($element,  $form_state, $form, $elementTypeManager, $definitions);
      }

      if ($isApplicable) {
        $isTypeExistant = isset($element['#type']) && isset($definitions[$element['#type']]);

        $type = 'default_element';

        if ($isTypeExistant) {
          $type = $element['#type'];
        }

        $instance = $elementTypeManager->createInstance($type);
        $instance->applyLogic($element);
      }
    }

    return $elements;
  }

}
